use crate::{
    AppError, layouts,
    models::{exercises::ExerciseVariant, workouts::WorkoutExerciseFull},
    util::AppState,
};
use axum::{
    Form, Router,
    extract::{Path, State},
    http::HeaderMap,
    response::IntoResponse,
    routing::get,
};
use maud::{Markup, html};
use serde::Deserialize;
use uuid::Uuid;

pub fn routes() -> Router<AppState> {
    Router::new()
        .route("/", get(page).delete(delete).put(submit_edit))
        .route("/edit", get(edit))
}

async fn page(State(state): State<AppState>, Path(id): Path<Uuid>) -> Result<Markup, AppError> {
    let workout = sqlx::query_as!(
        crate::models::Workout,
        "SELECT workout_id, name, description FROM workouts WHERE workout_id = $1",
        id
    )
    .fetch_one(&state.pool)
    .await?;

    let exercises = sqlx::query_as!(
        WorkoutExerciseFull,
        r#"
            SELECT exercises.exercise_id, exercise_type as "exercise_type:ExerciseVariant", sets, reps, time, name, description, position
            FROM workout_exercises
            JOIN exercises ON workout_exercises.exercise_id = exercises.exercise_id
            WHERE workout_id = $1
            ORDER BY position
        "#,
        &id,
    ).fetch_all(&state.pool).await?;

    tracing::info!("{:?}", exercises);

    let content = html! {
        div {
            div class="mb-5" {
                h2 { (workout.name) }
                p class="desc" { (workout.description) }
            }

            h3 { "Exercises" }

            div class="flex flex-col gap-y-3" {
                @for exercise in exercises {
                    (display_exercise(exercise))
                }
            }

            a href={"/workouts/" (id) "/start"} class="btn btn-success" { "Start Workout" }
        }
    };

    Ok(layouts::desktop(content, "View Workout", true))
}

fn display_exercise(exercise: WorkoutExerciseFull) -> Markup {
    html! {
        div class="bg-base-200 p-3 rounded flex" {
            div {
                h3 { (exercise.name) }
                p class="text-sm opacity-50" { (exercise.description) }
            }
            div class="ml-auto flex items-center" {
                @match exercise.exercise_type {
                    ExerciseVariant::Time => {
                        div { (exercise.time.unwrap_or(-1))"s" }
                    },
                    ExerciseVariant::Number => {
                        div {
                            div { (exercise.sets.unwrap_or(-1)) " sets" }
                            div { (exercise.reps.unwrap_or(-1)) " reps" }
                        }
                    },
                    ExerciseVariant::Failure => {
                        "Failure"
                    },
                }
            }
        }
    }
}

async fn delete(
    State(state): State<AppState>,
    Path(id): Path<Uuid>,
) -> Result<impl IntoResponse, AppError> {
    sqlx::query_as!(
        crate::models::Exercise,
        "DELETE FROM workouts WHERE workout_id = $1",
        id
    )
    .execute(&state.pool)
    .await?;

    let mut headers = HeaderMap::new();
    headers.insert("hx-location", "/workouts".parse().unwrap());

    Ok((headers, html! {}))
}

async fn edit(
    State(state): State<AppState>,
    Path(id): Path<Uuid>,
) -> Result<impl IntoResponse, AppError> {
    let exercise = sqlx::query_as!(
        crate::models::Exercise,
        "SELECT exercise_id, name, description FROM exercises WHERE exercise_id = $1",
        id
    )
    .fetch_one(&state.pool)
    .await?;

    let content = html! {
        h1 class="mb-5" { "Edit Exercise" }

        form hx-put={"/exercises/" (exercise.exercise_id)} class="space-y-1" {
            fieldset class="fieldset" {
                legend class="fieldset-legend" { "Name" }
                input required="true" name="name" class="input" value={(exercise.name)} {}
            }

            fieldset class="fieldset" {
                legend class="fieldset-legend" { "Description" }
                textarea required="true" name="description" class="textarea" { (exercise.description) }
            }

            input type="submit" class="btn" value="save" { }
        }
    };

    Ok(layouts::desktop(content, "Edit Exercises", true))
}

#[derive(Deserialize, Debug)]
struct FormData {
    name: String,
    description: String,
}

async fn submit_edit(
    State(state): State<AppState>,
    Path(id): Path<Uuid>,
    Form(form): Form<FormData>,
) -> Result<impl IntoResponse, AppError> {
    sqlx::query_as!(
        crate::models::Exercise,
        "UPDATE exercises SET name = $1, description = $2 WHERE exercise_id = $3",
        form.name,
        form.description,
        id
    )
    .execute(&state.pool)
    .await?;

    let mut headers = HeaderMap::new();

    let location = format!("/exercises/{}", id);
    headers.insert("hx-location", location.parse().unwrap());

    Ok((headers, html! {}))
}
